
//axes gebruiken.
/*
   box.cc -- standard routines for boxes.

   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
*/


#include "object.hh"
#include "shapes.hh"
#include "proto.hh"
#include "extern.hh"
#include "ibuf.hh"

shape_stats *rawbox = shape_stats::regist("raw box", -1);

int    box_data::do_intersection(ray * r, flt *hits, int ) const
{    
    rawbox->test--;       
    int n= intersect_rawbox( p1, p2, r, hits);
    if (n) {
	rawbox->hit--;
	vector p =    r->point(hits[0]);
	assert(p.length() < 10*INFTY);
    }
	
    return n;
}

void
box_data::bound(matrix *trans, bounding_box *bbox)
{
    do_box_bound(bbox, trans, p1, p2);
}

void
box_data::precompute()
{
    stat->howmuch++;
    vector  np1,
            np2;

    /* make sure the points are in the right order */
    for (int i= 0;i <3; i++){
	np1[i] = MIN(p1[i], p2[i]);
	np2[i] = MAX(p1[i], p2[i]);
    }


    p1 = np1;
    p2 = np2;
    prims++;
}

#include "matrix.hh"

void
do_box_bound(bounding_box *bbox, matrix *trans, vector p1, vector p2)
{
    bbox->set_empty();
    if (trans) {
	vector  point;
	point.set(p1.x, p1.y, p1.z);
	point = (*trans) * point;
	bbox->add(point);
	point.set(p2.x, p1.y, p1.z);
	point = (*trans) * point;
	bbox->add(point);
	point.set(p1.x, p2.y, p1.z);
	point = (*trans) * point;
	bbox->add(point);
	point.set(p1.x, p1.y, p2.z);
	point = (*trans) * point;
	bbox->add(point);
	point.set(p2.x, p2.y, p1.z);
	point = (*trans) * point;
	bbox->add(point);
	point.set(p1.x, p2.y, p2.z);
	point = (*trans) * point;
	bbox->add(point);
	point.set(p2.x, p1.y, p2.z);
	point = (*trans) * point;
	bbox->add(point);
	point.set(p2.x, p2.y, p2.z);
	point = (*trans) * point;
	bbox->add(point);
    } else {
	bbox->add(p1);
	bbox->add(p2);
    }
}

PUBLIC int
intersect_rawbox(vector p1, vector p2, ray * r, flt * hits)
{
    flt tmin = -INFTY;
    flt tmax = INFTY;
    rawbox->test++;
    
    /* intersect YZ-planes */
    for (int i = 0; i < 3; i++) {
	if (!ISZERO(r->dir[i])) {                           /* it's not parallel to YZ plane */
	    /* calculate two plane intersection,
	       optimised version We are still using the formula (x,n) = mov, but since n is (1,0,0),
	       calculations can be greatly simplified. */

	    flt t1 = (p1[i] - r->pos[i]) / r->dir[i];
	    flt t2 = (p2[i] - r->pos[i]) / r->dir[i];

	    if (t1 > t2) {                       /* swap t1 and t2 if t1 is not the minimum. */
		flt tmp = t1;
		t1 = t2;
		t2 = tmp;
	    }
	    
	    /* intersect previous domain with current */
	    if (tmin < t1)
		tmin = t1;
	    if (tmax > t2)
		tmax=t2;

	    // empty domain for rayparam?
	    if (tmax < tolerance || tmin > tmax)
		return 0;
	} else if (p1[i] > r->pos[i] || r->pos[i] > p2[i])
	    return 0;
    }

    /* OK. We've checked wether the ray r(t) is inside the box: the solutions set for t is not empty. both tmin and tmax are
     * intersection points of the ray with the box boundary (the 6 rectangles) Now choose the least positive of the two
     * intersection points, and return it. */

    rawbox->hit++;
    if (tmin < tolerance) {
	*hits = tmax;
	return 1;
    } else {
	hits[0] = tmin;
	hits[1] = tmax;
	return 2;
    }
}

bool
box_data::inside(vector org) const
{
    return (org.x < p2.x && org.y < p2.y && org.z < p2.z &&
            org.x > p1.x && org.y > p1.y && org.z > p1.z);
}

vector  box_data::normal(vector x) const
{
    vector  n;
    
    for (int i = X; i <= Z; i++)
	n[i] = -ISZERO(x[i] - p1[i]) + ISZERO(x[i] - p2[i]);


    
    if (!bool(n)) {
	p1.print("p1");
	p2.print("p1");
	n.print("n");
	x.print("x");
    }
    assert(bool(n));
    return n;
}

shape_stats *box_data::stat = shape_stats::regist("box");
box_data::box_data(vector a,vector b)
{
    type = BOX;
    p1=a;
    p2=b;
}

void
box_data::scale(const vector s)
{
    p1.scale(s);
    p2.scale(s);
}

void
box_data::translate(const vector t)
{
    p1 += t;
    p2 += t;
}

void
box_data::print() const
{
#ifdef DEBUG
    p1.print("p1");
    p2.print("p2");
#endif          /* DEBUG */
}

PUBLIC object *
        get_new_box_object(vector a,vector b)
{
    object *o = new object;
    o->data = new box_data(a,b);
    o->data->setdaddy(o);
    o->flags = O_CLOSED | O_FINITE_INSIDE ;

    return o;
}
