/*
   color.cc -- colors.

   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
 */


#include "proto.hh"
#include "extern.hh"
#include "color.hh"

//  multiply colors, by components
void
nc_color:: operator *= (const nc_color & c1)
{
    x *= c1.x;
    y *= c1.y;
    z *= c1.z;
}

nc_color operator *(const nc_color & c1, const nc_color & c2)
{
    nc_color c = c1;
    c *= c2;
    return c;
}


void
nc_color::print() const
{
#ifdef DEBUG
    dprintf("r %f g %f b %f\n", x, y, z);
#endif          /* DEBUG */
}
/*
  a norm on the color triples. The formula used is .299 r + .587 g + .114
  b (from the PBM utils)
 */
// works for darklight
flt     color::length()
{
    return 0.299 * abs(r()) + 0.587 * abs(g()) + 0.114 * abs(b());
}

/*
 * normalize a color: make it fit in [0,1]
 */
void
color::normalise()
{   
    if (x < 0)
        x = 0;

    if (y < 0)
        y = 0;

    if (z < 0)
        z = 0;

    flt     m = MAX(r(), g());
    m = MAX(m, b());
    if (m > 1)
        *this /= m;
}
