/*
   composite.c --  routines for composites

   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
 */


#include "object.hh"
#include "nonprim.hh"
#include "proto.hh"
#include "extern.hh"
#include "ibuf.hh"
#include "texture.hh"
#include "opti.hh"

void
composite_data::add_children_to_optilist(Optilist *l)
{
    for (object_iter p(contents); p; p.next())
	p->add_to_optilist(l);
}

bool
composite_data::all_intersections(ibuf * globq, ray * r, int flags, bool & ins) const
{
    bool    gotcha;
    stat->test++;
    gotcha = optidata->all_intersections(globq, r, flags, ins);
    if (gotcha)
        stat->hit++;

    return gotcha;
}

/* add speed to the composite */
void
composite_data::add_speed(vector spd)
{
    contents.add_speed(spd);
}
/* initialize a composite_data struct */

shape_stats *composite_data::stat = shape_stats::regist("composite");

composite_data::composite_data()
{
    type = COMPOSITE;
    otype = NONE;
    optidata = 0;
}

/* append an object to the composite c */
void
composite_data::add_child(object * o)
{
    o->daddy =daddy;
    contents.add_to(o);
}

/* this is getting boring... */
void
composite_data::translate(const vector t)
{
    contents.translate(t);
}

/* hooooeeewaahhh, Geez, why do I write these comments? */
void
composite_data::rotate(const matrix & rotmat)
{
    contents.rotate(rotmat);
}

/* 3X raden (Yes, you folks out there, I'm now commenting in Dutch! */
void
composite_data::scale(const vector s)
{
    contents.scale(s);
}

/*
 * I guess .. because somebody will read them, some day (Hello, is there
 * somebody out there?
 *
 * and another destructor (en wat hebben ze gewonnen, Pierre, als ze goed
 * raden waar dit voor is ?
 */

composite_data::~composite_data()
{

}

/* do debugging stuff */
void
composite_data::print() const
{
#ifdef DEBUG
    dprintf("optimisation %d \n", otype);
    contents.print();
    if (optidata)
      optidata->print();
#endif          /* DEBUG */
}

/* init optimisation routines */
void
composite_data::bound(matrix *, bounding_box *bbox)
{
    if (otype == NONE)
	contents.bound();

    bbox->set_empty();

    /* for the data: union of bboxes */
    if (!(daddy->flags & O_INFINITE))
	for (object_iter op(contents); op.ok(); op.next()){
	    if (op->flags & O_INVIS)
		continue;
	    bbox->add(*op->bbox);
	}
}


PUBLIC object *
        get_new_composite_object()
{
    object *o;

    composite_data *c = new composite_data;
    o = new object;
    c->setdaddy(o);
    o->data = c;

    return o;
}


/****************************************************************
  stupid raytracing
 ****************************************************************/

struct  vanilla_optimisation :object_list_optimisation {
    const object_list *l;
    bool all_intersections (ibuf * q, ray * r, int flags, bool & b) const;
    void print() const;
    vanilla_optimisation(object_list const & il) { l = &il; }
};

/* print some nonformation */
void vanilla_optimisation::print() const
{
    dprintf("no optimisation\n");
}


object_list_optimisation::~object_list_optimisation() {}

/* exhaustive raytracing */
bool
vanilla_optimisation::all_intersections(ibuf * globq, ray * r, int flags, bool &ins) const
{
    bool        gotcha = false;

    for (object_iter p(*l); p.ok(); p.next())
    {
        /* walk the element list, and intersect each element */
        if (p->all_intersections(globq, r, flags, ins))
            gotcha = true;

	flt             mint = globq->mint();
        if (!(flags & CHKALL))
            r->maxt = mint;

        if (mint < r->mint)     /* early exit possible? */
            break;
    }
    return gotcha;
}


void
composite_data::precompute()
{
    stat->howmuch++;
    contents.precompute(daddy);

    daddy->flags &= ~O_INFINITE;
    for (object_iter op(contents); op.ok(); op.next())
    {
        if (op->flags & O_INFINITE) {
            daddy->flags |= O_INFINITE;
	    break;
        }
    }

    switch (otype)
    {
    case NONE:
        optidata = new vanilla_optimisation(contents);
        break;
    case BSP:
	{
	    contents.bound();
	    Optilist k(this);
	    optidata = init_BSP(k);
	}
	break;

    default:
        errormsg("unknown optimisation");
    }

    nonprims++;
}

/* hoihoihoihoi. */
/* einde . Ik word gefrotofrageerd. */
/****************************************************************/
void
Scene::set_root_object() {
    object *p = get_new_composite_object();
    p->text = new simple_texture;
    the_object = p;
}

void
Scene::set_root_optimisation(int i) {
    optitype  t = (optitype) i;
    ((composite_data *)the_object->data)->otype = t;
}



