/*
   keyword.cc -- handle declarations and  keywords.
 */
#include "ray.hh"
#include "parse.hh"
#include "extern.hh"
#include "proto.hh"
#include "texture.hh"
#include "camera.hh"
#include "color.hh"
#include "object.hh"

#if defined MSDOS || defined __MSDOS__ || defined msdos || defined __msdos__
#include "rayparse.hh"
#else
#include "rayparse.tab.hh"
#endif


/***************************************************************************
 * one declaration							   *
 ***************************************************************************/

Identifier::Identifier(const char *s)
{
    name = xstrdup(s);
    type = IDENTIFIER;
    data = 0;
}

void
Identifier::del_data()
{
    if (data)
        switch (type)
        {
        case IDENTIFIER:
            assert(false);
        case FLOAT_IDENTIFIER:
            delete(flt *) data;
            break;
        case TEXTURE_IDENTIFIER:
            delete(texture *) data;
            break;
        case IMAGEMAP_IDENTIFIER:
            delete(image_map *) (data);
            break;
        case COLOR_IDENTIFIER:
            delete(nc_color *) data;
            break;
        case CAMERA_IDENTIFIER:
            delete(Camera *) data;
            break;

        case VECTOR_IDENTIFIER:
            delete(vector *) data;
            break;

        case OBJECT_IDENTIFIER:
            delete(object *) data;
            break;
        default:
            assert(false);
        }
    data = 0;
    type = IDENTIFIER;
}

/* kill memory used by declarations */
Identifier::~Identifier()
{
    delete[]  name;
    del_data();
}

void
Identifier::print() const
{
#ifdef DEBUG
    dprintf("\n\"%s\": \n", name);
    switch (type)
    {
    case IDENTIFIER:
        /* assert(false); */
        /* tussentijds printen mogelijk */
        dprintf("identifier\n");
        break;
    case FLOAT_IDENTIFIER:
        dprintf("%f\n", *(flt *) data);
        break;
    case COLOR_IDENTIFIER:
        ((color *) data)->print();
        break;
    case VECTOR_IDENTIFIER:
        ((vector *) data)->print("");
        break;
    case TEXTURE_IDENTIFIER:
        ((texture *) data)->print();
        break;
    case IMAGEMAP_IDENTIFIER:
        ((image_map *) data)->print();
        break;
    case CAMERA_IDENTIFIER:
        ((Camera *) data)->print();
        break;
    case OBJECT_IDENTIFIER:
        ((object *) data)->print();
        break;
    default:
        assert(false);
    }
#endif          /* DEBUG */
}


/*******************************************************
   all declarations
 *******************************************************/
class Declare_tab
{
    Identifier *first;
public:
            Declare_tab();
            ~Declare_tab();
    Identifier *lookup(const char *s);
    void print() const ;
}      *the_dectab;

Declare_tab::Declare_tab()
{
    first = 0;
}

Declare_tab::~Declare_tab()
{
    for (Identifier *p = first, *next = 0; p; p = next){
        next = p->next;
        delete  p;
    }
    first = 0;
}
//
void
Declare_tab::print() const
{
#ifdef DEBUG
    Identifier *p;
    for (p = first; p; p = p->next)
    {
        p->print();
    }
#endif
}
// lookup s, and create new ident. if not found
Identifier *
        Declare_tab::lookup(const char *s)
{
    Identifier *p;
    for (p = first; p; p = p->next)
        if (!strcmp(p->name, s))
        {
            break;
        }
    if (!p) {
	p = new Identifier(s);
	p->next = first;
	first = p;

    }
    return p;
}

Identifier * lookup_declaration(const char *s)
{
    if (!the_dectab)
	the_dectab = new Declare_tab;
    return the_dectab->lookup(s);
	    
}
void kill_declarations()
{
 #ifdef DEBUG
    if (debug_options & DEBUGDECL)
        the_dectab->print();
#endif
 
   delete the_dectab;
}

/****************************************************************
  KEYWORDS
 ****************************************************************/

struct Keyword_ent
{                               /* for the keyword table */
    const char   *name;
    int     tokcode;
};

class Keyword_table
{
    Keyword_ent *table;
    int     maxkey;
public:
    Keyword_table(Keyword_ent *);
    int     lookup(const char *s) const ;
};


/* for qsort */
int
        tabcmp(const void * p1, const void * p2)
{
    return strcmp(((const Keyword_ent *) p1)->name,
                  ((const Keyword_ent *) p2)->name);
}

#include "toktab.hh"

static Keyword_table *the_keytab  = 0;

int
lookup_keyword(const char *s) {
    if (!the_keytab)
	the_keytab = new Keyword_table(my_tab);
    return the_keytab->lookup(s);
}

Keyword_table::Keyword_table(     Keyword_ent *myt)
{
    table = myt;

    /* count keywords */
    for (maxkey = 0; table[maxkey].name; maxkey++);

    /* sort them */
    qsort(table, maxkey, sizeof(Keyword_ent), tabcmp);
}

// lookup with binsearch, return tokencode.
// should be using hash..
// Who cares?
int
        Keyword_table::lookup(const char *s) const 
{
    int     lo,
            hi,
            cmp,
            result;
    lo = 0;
    hi = maxkey;

    /* binary search */
    do
    {
        cmp = (lo + hi) / 2;

        result = strcmp(s, table[cmp].name);

        if (result < 0)
            hi = cmp;
        else
            lo = cmp;
    }
    while (hi - lo > 1);
    if (!strcmp(s, table[lo].name))
    {
        return table[lo].tokcode;
    } else
        return -1;              /* not found */
}
