/*
   main.c -- main() and its belongings.

   based on work originally by George Kyriazis, 1988.

   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
*/

#ifdef __GNUC__
#include <unistd.h>
#endif

#include <string.h>

#include	"proto.hh"
#include 	"extern.hh"
#include 	"object.hh"
#include 	"camera.hh"
#include 	"lgetopt.hh"

static void usage();
static void notice();


static char    *infn = xstrdup( "input.r");
static char    *outfn = xstrdup("output.tga");

void scan_command_line(int argc, char *argv[])
{    
    long_option_init the_opts[] = {
	{false, "continue", 'c'},
	{true, "debug", 'd'},
	{true, "height", 'h' },
	{true, "width", 'w' },
	{false, "disable-exit", 'x' },
	{false, "test", 't' },
	{true, "include", 'l' },
	{true, "iterations", 'I' },
	{false, "help", '?' },
	{true, "input", 'i' },
	{true, "output", 'o' },
	{0,0}
    };
    Getopt_long parser( argc, argv, the_opts);
    
    while (1) {
	long_option_init *op = parser();	    
	if (!op)
	    break;
	char c = op->shortname;

        switch (c)
        {
        case 'c':               /* continue a previously interrupted picture */
            continue_flag = true;
            break;

        case 'd':               /* turn on debugging */
            debug_options = parser.intarg();
            if (!debug_options)
                debug_options = ~0;
            break;

        case 's':
            silent_mode = true;
            break;
        case 'h':               /* vertical  resolution */
            yres = parser.intarg();
            break;
        case 'w':               /* horizontal resolution */
            xres = parser.intarg();
            break;
        case 'x':               /* disable keypress-exit */
            keyboard_exit = true;
            break;
        case 't':               /* for testing 'n debugging */
            test_flag = true;
            break;
        case 'l':               /* library path */
            add_path(parser.optarg);
            break;
        case 'I':               /* number of frames, "iterations" */
            iter_override = parser.intarg();
            the_scene.tries = iter_override;
            break;
        case 'o':               /* output file */
	    delete[] outfn;
            outfn= xstrdup(parser.optarg);
            break;
        case 'i':               /* inputfile */
	    delete[] infn;
            infn = xstrdup(parser.optarg);
            break;
        case '?':
            usage();
            break;
        default:
            assert(false);
        }
    }
}

int
main(int argc, char *argv[])
{
    /* set some global vars !!-->NEEDED FOR dprintf OUTPUT<-- */
    initialize();

    /* print the (c) stuff */
    notice();

    /* no commandline options? */
    if (argc == 1)
        usage();
    
    scan_command_line(argc, argv);

    /* initialize any machine dependent stuff */
    init_machine();

    /* parse the input */
    the_scene.read_from(infn);
    the_scene.precompute();
    
    if (debug_options & DEBUGINTERIOR) {    
	the_scene.print();
    }
    
    /* and do the tracing */
    the_scene.render_to(outfn);
    exit(0);
}

/* print help screen */
static void
usage()
{
    printf("Compiled on " __DATE__ "\n"
           "Rayce <options>\n"
           "-I, --iterations #  number of frames\n"
           "-i, --input <file>  input\n"
           "-o, --output <file> output\n"
           "-h, --height	height\n"
           "-w, --width		width\n"
           "-x, --disable-exit  disable exit\n"
           "-c, --continue      continue trace\n"
           "-d, --debug #       debugging (needs -DDEBUG with compiling)\n"
           "-t, --test          testing\n"
	   "-?, --help	        help\n"
           "-l, --include path  add library path\n"
           "  1 = tokenizer, 2 = parser, 4 = runtime warnings, \n"
           "  8 = print interior, 16 = print defines 32 = selfcheck\n"
           "\n"
           "  Rayce was compiled with DEBUG %sdefined\n",

#ifdef DEBUG
           ""
#else
           "not "
#endif          /* DEBUG */

        );
    exit(0);
}

static void 
notice()
{
    printf("This is Rayce version 3.0 beta\n"
           "Rayce is (c) 1993, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>\n"
           "  It is based on \"Ray\" by George Kyriazis <kyriazis@turing.cs.rpi.edu>\n"
           "  Rayce is released under the terms of the GNU General Public License,\n"
           "  it has no warranty what so ever. For more details: see the file \"COPYING\"\n"
           "\n"
           "Enjoy!\n\n");

}
