#ifndef NONPRIM_HH
#define NONPRIM_HH

#include "stats.hh"
#include "object.hh"
#include "shapeheads.hh"

/// types of optimisation 
enum optitype
{
    NONE,                       /* use parent's optimisation */
//    MANUAL,                     /* user specified BVH */
    BSP,                        /* build BSP of contents */
};

void set_root_optimisation (optitype t);

struct object_list_optimisation {
    virtual     bool all_intersections (ibuf * q, ray * r, int flags, bool & b) const = 0;
    virtual     void print() const = 0;
    virtual ~object_list_optimisation();
};

struct composite_data : shape_data {
    MAKESHAPE(composite);
    virtual void add_child(object *);

    optitype otype;
    void bound(matrix *,bounding_box *m);
    composite_data();
    
    ~composite_data();
    void add_children_to_optilist(Optilist *);
    void rotate(matrix const & rotmat);
    void translate(const vector t);
    void scale(const vector s);
    void add_speed(vector spd);
    bool all_intersections(ibuf * q, ray * r, int flags, bool & b) const;
private:

    object_list_optimisation *optidata;
    object_list contents;
};

struct CSG_data : shape_data
{

    void add_child(object * o);
    virtual ~CSG_data() {}
    void rotate(matrix const & rotmat);
    void translate(const vector t);
    void scale(const vector s);
    virtual bool inside(vector) const = 0;
protected:
    object_list contents;
};

struct inter_data : public CSG_data
{
    void bound(matrix *,bounding_box *m);
    bool all_intersections(ibuf * q, ray * r, int flags, bool & b) const;
    shape_data *clone () { return new inter_data (*this); }
    SHAPEMETHODS();
    bool inside(vector) const;
};

struct union_data : public CSG_data
{
        void bound(matrix *,bounding_box *m);
    bool all_intersections(ibuf * q, ray * r, int flags, bool & b) const;
    shape_data *clone () { return new union_data (*this); }
    SHAPEMETHODS();
    bool inside(vector) const;
};


#endif
