
#include "object.hh"
/****************************************************************
  LISTS OF OBJECTS
 ****************************************************************/

object_list::object_list()
{
    first = 0;
}

void
object_list::append_to(object * toadd)
{
    assert(!toadd ->next); /* can't be member of other list */
    if (empty())
	first = toadd;
    else {
	object *last = 0;
	for (object * o = first; o; o = o->next)
	    last = o;

	last->next = toadd;
    }
}

void
object_list::add_to(object * toadd)
{
    assert(!toadd->next); /* can't be member of other list */
    toadd->next = first;
    first = toadd;
}

object_list::object_list(object_list &src)
{
    first =  0;
    append_to(&src);
}


void
object_list::operator =(object_list &src)
{
    for (object *p = first, *next = 0; p ; p = next) {
	next = p->next;
	delete p;
    }
    first = 0;
    append_to(&src);
}


void
object_list::append_to(object_list * t)
{
    if (t->empty() )
	return;

    assert(t != this);
    
    object *start_src, *start_dst;

    if (empty()) {	
	first = start_dst = new object(*t->first);
	start_src = t->first->next;
    } else {
	start_src = t->first;
	for (object * o = first; o; o = o->next)
	    start_dst = o;
	assert(start_dst);
    }
    
    for (object *p = start_dst, *q = start_src; q; q = q->next) {	
	p->next = new object(*q);
	p = p->next;
    }
}

void
object_list::add_speed(const vector s)
{
    for (object * start = first; start; start = start->next)
        start->add_speed(s);
}

void
object_list::precompute(object *daddy)
{
    for (object * o = first; o; o = o->next) {
      o->daddy = daddy;
      o->precompute();	
    }
}

void
object_list::bound()
{
    for (object * o = first; o; o = o->next)
        o->bound();
}

object_list::~object_list()
{
    for (object *next = 0, *p = first; p; p = next) {
        next = p->next;
        delete(p);
    }
}

void
object_list::rotate(const matrix & rotmat)
{    
    for (object * p =first; p; p = p->next)
        p->rotate(rotmat);
}

void
object_list::scale(vector r)
{
    for (object * p = first; p; p = p->next)
        p->scale(r);
}


void
object_list::translate(vector r)
{
    for (object * p = first; p; p = p->next)
        p->translate(r);
}

void
object_list::print() const
{
#ifdef DEBUG
    dprintf("[");
    for (object *o = first; o; o = o->next)
        o->print();
    dprintf("]\n");
#endif          /* DEBUG */
}

