#include 	"extern.hh"
#include "object.hh"
#include "color.hh"
#include "light.hh"
#include "nonprim.hh"
#include "texture.hh"
#include 	"grid.hh"
#include 	"camera.hh"

Scene::~Scene() {
    delete the_object;
    delete the_cam;
    delete global_ior;
    delete ambient_light;
    delete background_color;
    delete the_grid;
}

/// do some problem-wide constants 
Scene::Scene() {
    background_color = new Color(0.0);
    bggradient.set(0.0);
    max_trace_level = DEFAULTMAXLEVEL;
    ambient_light = new Color(1);
    global_ior = 0;
    ior_depth = 0;
    the_grid = new Grid;
    atmosphere_ior = 1;
    tolerance = DEFAULT_TOLERANCE;

    bunching = 10;    
    set_root_object();
    set_root_optimisation(BSP);
    the_cam = new Camera;
    the_output = 0;

    start = stop = 0.0;
    tries = 1;
}
void Scene::set_bg(color c,vector gr )
{
    *background_color = c;
    bggradient = gr;
}
void Scene::set_filter(int i)
{
    the_grid->filter_type = i;
}

void Scene::set_jitter(flt d) {
    the_grid->jitter_width_x=d;
    the_grid->jitter_width_y = d;
}

void
Scene::set_time(flt t1, flt t2)
{
    start = t1; stop = t2;
}
void
Scene::add(object *p)
{
    no_obs ++;
     the_object->data->add_child (p);
}

void
Scene::set_cam(Camera *c)
{
    delete the_cam;
    the_cam = c;
}

void
Scene::set_text(texture *p)
{
    delete    	  the_object->text ;
    the_object->text = p;
}

/// precompute global stuff, called AFTER parsing 
void
Scene::precompute()
{
    /* should init'ed after parsing, because of maxtracelevel */
    global_ior = new flt[max_trace_level + 2];
    global_ior[0] = global_ior[1] = atmosphere_ior;
    ior_depth = 1;
    printf("Precomputing..\n");

    the_object->precompute();
    the_grid->precompute();
    the_cam->precompute();
    delta_t = (stop-start) / tries;
}

/// pick Time inside time interval.
flt
Scene::pick_time(int i)
{
  return start + (i + frand()) * delta_t;
}
/** Randomization is used so we won't get the strobo effect */

/// called BEFORE parsing, BEFORE scanning cmd line.
void
initialize()
{
    error_out = stdout;

    /* some defaults */
    xres = yres = 50;

    debug_options = 0x00;
    keyboard_exit = true;
    continue_flag = false;
    silent_mode = false;
}

void
Scene::print() const
{
#ifdef DEBUG
    the_cam->print();
    dprintf("time: [%f, %f], tolerance %f\n", start, stop, tolerance);
    dprintf("frames %d,  recursion %d\n", tries, max_trace_level);
    dprintf("cutoff %f,", depth_cutoff);
    bggradient.print("background gradient");
    dprintf("bg color:");
    background_color->print();
    dprintf("atmosphere {\n ior %f, ", atmosphere_ior);
    ambient_light->print();
    dprintf("}\n");

    dprintf("\n=================SCENE ELEMENTS==================\n");
    the_object->print();
    dprintf("\n==================LIGHT LIST=====================\n");
    light_data::print_list();
    dprintf("\n==================FOG LIST=====================\n");
    fog_data::print_list();
    dprintf("\n====================GRID=========================\n");
    the_grid->print();
#else
    printf("Debug not def'd\n");
#endif
}

