#include "extern.hh"
#include "ray.hh"
#include "ibuf.hh"
#include "stats.hh"
#include "object.hh"

void
shape_stats::start()
{
    printf("%ld primitives in scene, %ld non primitives\n", shape_data::prims, shape_data::nonprims);
    timer_start();
}

#include        "vray.hh"
static svec<shape_stats*> *stat_table = 0;


/*
  DESCRIPTION
  enable stat printing.

  PARAMS
  s:	name
  h:	initial number ( h < 0 -- no howmuch print)

  RETURN
  pointer to reserved statistic struct
*/
shape_stats *shape_stats::regist(const char *s, int h )
{
    if (!stat_table)
	stat_table = new svec<shape_stats*>;

    shape_stats *a = new shape_stats(s, h);
    (stat_table)->add(a);

    return a;
}

#define FMT0 " %-23s\t%12s\t%12s\t%13s\n"
#define FMT1 " %-23s\t%12ld\t%12ld"
#define FMT2 " %-17s%6ld\t%12ld\t%12ld"
void
shape_stats::print_header()
{
    printf(FMT0, "", "tested", "succeeded", "relative");
}

shape_stats::shape_stats(const char *s, int h)
{
    hit =  test = 0;
    howmuch =h;
    name = xstrdup(s);
}

void
shape_stats::print() const
{
    long int h = howmuch;
    if (!howmuch)
    {
        if (debug_options & DEBUGRUNTIME)
            h = -1;
        else
            return ;
    }
    if (h > 0)
        printf(FMT2, name, h, test, hit);
    else if (h < 0)
        printf(FMT1, name, test, hit);

    if (test > 0)
        printf("\t%12.2f%%\n", 100.0 * (flt) hit / (flt) test);
    else
        printf("\n");
}

void
shape_stats::print_all()
{
    print_header();

    for (int i = 0; i < (*stat_table).sz(); i++)
        (*stat_table)[i]->print();

    long    sec = timer_stop();
    printf("\n");
    printf("Total reflected rays traced: %ld\n", ray::stats.reflected);
    printf("Total refracted rays traced: %ld\n", ray::stats.refracted);
    printf("Total eye rays traced: %ld\n", ray::stats.eye);
    printf("Total shadow rays traced: %ld\n", ray::stats.shadow);

    if (test_stat)
        printf("\n\nTEST STATISTICS: %ld\n", test_stat);

    printf(" Time for trace: %d:%02d:%02d (%ld seconds)", (int) sec / 36000,
           (int) (sec / 600) % 60, (int) (sec % 600) / 10, (long) sec / 10);
    if (ray::stats.rays) { // debug ctors
	if (ray::stats.rays->test > 0)
	    printf(" Time per ray: %.3fms\n",
		   100.0 * (flt) sec / ray::stats.rays->test);      /* 100 = 10/mili */
	else
	    printf("\n");
    }						 

}
